import pygst
pygst.require('0.10')
import gst
import time
import gobject

gobject.threads_init()

class Camera:
  def __init__ (self):
    pipeline = gst.parse_launch ("""
      v4l2src ! tee ! queue max-size-buffers=1 leaky=1 name=queue !
        ffmpegcolorspace ! video/x-raw-rgb,depth=24,bpp=24 ! 
        fakesink signal-handoffs=1 name=fakesink"""
    )

    self.fake_sink  = pipeline.get_by_name ('fakesink')
    self.pipeline = pipeline

  # public methods

  def turnOff (self):
    self.pipeline.set_state (gst.STATE_PAUSED)

  def takePicture (self):
    self.buffer = None
    self.takePictureAsync (None)
    while (self.buffer == None):
      time.sleep (0.25)
    return self.result

  def takePictureAsync (self, callback):
    self.pictureCallback = callback
    self.probe_id = self.fake_sink.get_pad ('sink').add_buffer_probe (self._bufferHandler, None)
    self.pipeline.set_state (gst.STATE_PLAYING)

  # private methods

  def _bufferHandler (self, pad, buffer, data):
    self.fake_sink.get_pad ('sink').remove_buffer_probe (self.probe_id)
    self._pictureResult (buffer)

    if self.pictureCallback is not None:
      self.pictureCallback (self.result)

    self.pictureCallback = None

  def _pictureResult (self, buffer):
    caps = buffer.caps 
    self.width = caps[0]["width"] 
    self.height = caps[0]["height"] 
    self.buffer = buffer
    self.result = [self.buffer, self.width, self.height]
